use super::*;

fn dump_node_tree_helper(lna: LayoutNodeAccess, indent: usize, out: &mut String) {
    let ind = "    ".repeat(indent);
    out.push_str(ind.as_str());
    out.push_str("Node (");
    out.push_str(match lna.label() {
        Some(v) => v,
        None => "<anon>",
    });
    out.push_str(
        format!(
            ") [wpol: {}/{}/{} hpol: {}/{}/{} behaviour: {:?} upd: {:?} rend: {:?}]\n",
            lna.width_policy.minimum,
            lna.width_policy.desired,
            lna.width_policy.slack_weight,
            lna.height_policy.minimum,
            lna.height_policy.desired,
            lna.height_policy.slack_weight,
            lna.behaviour,
            lna.with_state(|v| v.needs_update),
            lna.with_state(|v| v.needs_render),
        )
        .as_str(),
    );
    out.push_str(ind.as_str());
    out.push_str("        ");
    out.push_str(format!("net policy:  {:?}\n", lna.with_state(|v| v.net_policy)).as_str());
    out.push_str(ind.as_str());
    out.push_str("        ");
    out.push_str(format!("render area: {:?}\n", lna.render_area()).as_str());

    for child in lna.child_iter() {
        dump_node_tree_helper(child, indent + 1, out);
    }
}

pub fn dump_node_tree(lna: LayoutNodeAccess, out: &mut String) {
    dump_node_tree_helper(lna, 0, out);
}

pub fn dump_tree_json<W: ?Sized + std::io::Write>(
    lna: LayoutNodeAccess,
    out: &mut std::io::BufWriter<W>,
) -> std::io::Result<()> {
    use std::io::Write;

    write!(out, "{{")?;
    write!(out, "\"label\": \"{}\" ", lna.label().unwrap_or("null"))?;
    write!(
        out,
        ",\"id\": {} ",
        <LayoutNodeID as Into<usize>>::into(lna.id())
    )?;
    write!(out, ",\"behaviour\": \"{:?}\"", lna.behaviour())?;
    write!(out, ",\"child_arrangement\": \"{:?}\"", lna.arrangement())?;
    write!(
        out,
        ",\"width_policy\": [{},{},{}]",
        lna.width_policy().minimum,
        lna.width_policy().desired,
        lna.width_policy().slack_weight
    )?;
    write!(
        out,
        ",\"height_policy\": [{},{},{}]",
        lna.height_policy().minimum,
        lna.height_policy().desired,
        lna.height_policy().slack_weight
    )?;
    write!(out, ",\"halign\": \"{:?}\"", lna.halign())?;
    write!(out, ",\"valign\": \"{:?}\"", lna.valign())?;
    write!(
        out,
        ",\"margin\": {{\"top\": {}, \"bottom\": {}, \"left\": {}, \"right\": {}}}",
        lna.margin().top,
        lna.margin().bottom,
        lna.margin().left,
        lna.margin().right,
    )?;
    match lna.table_cell() {
        None => write!(out, ",\"table_cell\": null")?,
        Some(c) => write!(out, ",\"table_cell\": [{},{}]", c.x, c.y)?,
    }
    match lna.render_area() {
        None => write!(out, ",\"render_area\": null")?,
        Some(a) => write!(
            out,
            ",\"render_area\": [[{},{}],[{},{}]]",
            a.min.x, a.min.y, a.max.x, a.max.y
        )?,
    }

    write!(out, ",\"children\": [")?;

    let mut needs_sep = false;
    for child in lna.child_iter() {
        if needs_sep {
            out.write(",".as_bytes())?;
        } else {
            needs_sep = true;
        }
        dump_tree_json(child, out)?;
    }

    write!(out, r#"]}}"#)?;

    Ok(())
}
