#![doc = include_str!("../README.md")]

pub(crate) mod render;

pub mod font;
pub mod input;
pub mod layout;
pub mod theme;
pub mod ui;
pub mod widget;
pub mod window;

/// Commonly-used traits collected in one place for easier imports.
pub mod prelude {
    pub use crate::widget::{Widget, WidgetExt};
    pub use crate::window::WindowComponent;
    pub use crate::Component;
}

/// Crates with types exposed in the patina API.
pub mod re_exports {
    pub use fontdue;
    pub use kahlo;
}

/// Math types for representing points, sizes, and rectangular regions.
pub mod math {
    pub use kahlo::math::*;
}

/// Trait for any object that participates in the event handling tree.
pub trait Component {
    /// What type of message to pass up the tree?
    type ParentMsg;
    /// What type of message do we know how to process?
    type Msg;

    /// Process a single incoming message and return the resulting messages for the parent
    /// component.
    fn process(&mut self, msg: Self::Msg) -> Vec<Self::ParentMsg>;

    /// Process multiple incoming messages and return the resulting messages for the parent
    /// component.
    fn process_all(&mut self, msgs: impl Iterator<Item = Self::Msg>) -> Vec<Self::ParentMsg> {
        msgs.flat_map(|msg| self.process(msg)).collect()
    }
}
