use std::ops::DerefMut;

use crate::{
    input::InputState,
    layout::{LayoutNode, LayoutNodeAccess, LeafLayoutNode, SizePolicy},
    render::RenderTarget,
    ui::UIHandle,
    widget::Widget,
    Component,
};

pub struct Spacer<C: Component> {
    lnode: LeafLayoutNode,
    _ghost: std::marker::PhantomData<C>,
}

impl<C: Component> Spacer<C> {
    pub fn new(uih: &UIHandle) -> Self {
        let mut lnode = LeafLayoutNode::new(uih.new_layout_node());
        lnode
            .set_width_policy(SizePolicy::expanding(1))
            .set_height_policy(SizePolicy::expanding(1));
        Self {
            lnode,
            _ghost: Default::default(),
        }
    }
}

impl<C: Component> Widget<C> for Spacer<C> {
    fn poll(
        &mut self,
        _uih: &mut UIHandle,
        _input_state: Option<&InputState>,
    ) -> Vec<<C as Component>::Msg> {
        vec![]
    }

    fn layout_node(&self) -> LayoutNodeAccess {
        LayoutNodeAccess::new(&self.lnode)
    }
    fn layout_node_mut(&mut self) -> &mut LayoutNode {
        self.lnode.deref_mut()
    }

    fn render<'r, 'data: 'r>(&self, target: &mut RenderTarget<'r, 'data>) {
        target
            .with_node(&self.layout_node())
            .fill(crate::render::ColourChoice::Background)
            .finish();
        /*(let region = self.layout_node().render_area().unwrap();
        if self.lnode.render_check() {
            target.fill_region(region, theme.background);
        }*/
    }
}
